#define __cplusplus__
#ifndef __MOSES1__
#define __MOSES1__

#include <cmath>
#include <iostream>
#include <cstdlib>
#include <ctime>

#include "Function.hpp"


#define deux_pi (2.0*M_PI)


class MOSES1
{
private:
  int _taille_pop;
  double _inf;
  double _sup;
  double **pop;
  int _dim;
  // double *optimum;
  double f_arret , S, S2;
  double pmut, rayon;
  Function* _F;
  double *optimum;
  double *values;
  double f_min;
  double I_min;
  long int evaluations;
  int nb_mutants;
  int Imin;

public:

  MOSES1();
  MOSES1(Function *F,  int n);
  ~MOSES1();
  double getOptimum();

  void set_dim(int d);
  void set_Rad(double r);
  void set_Size(int n);
  void set_pmut(double p);
  void set_arret(double arr);
  void set_fmin(double fm);

  void start(bool view, long int nb_eval);
  // void minimum(double *v);
  void mutation();
  void selection();
  int binom(int n, double p);
  void init();
  double f(double *x);
  int search_min_pop();
  void info();
};


//**************************************************************************

MOSES1::MOSES1(){

  _taille_pop=8;
  f_arret=0.0001; 
  pmut=0.4;
  rayon=0.1;
  evaluations=0;
  _F=new Function();
  _inf=_F->getXmin();
  _sup=_F->getXmax();
  _dim=_F->get_dim();

}

//***************************************************************************************
MOSES1::MOSES1(Function* F, int n){

  _taille_pop=n;
  f_arret=0.0001; 
  pmut=0.4;
  rayon=0.1;
  evaluations=0;
  _F=F;
  _inf=_F->getXmin();
  _sup=_F->getXmax();
  _dim=_F->get_dim();
}

//**************************************************************************

MOSES1::~MOSES1(){
  delete [] values;
  delete [] optimum;
  for(int i=0;i<_taille_pop;i++) delete [] pop[i];
  delete [] pop;
  

}

//**************************************************************************

void MOSES1::set_dim(int d){
  _dim=d;
}

void MOSES1::set_Rad(double r){
  rayon=r;
}

void MOSES1::set_Size(int n){
  _taille_pop=n;
}

void MOSES1::set_pmut(double p){

  pmut=p;
}

void MOSES1::set_arret(double arr){

  f_arret=arr;
}

void MOSES1::set_fmin(double fm){

  f_min=fm;
}



//**************************************************************************
double MOSES1::getOptimum(){
  return(f_min);
}

//**************************************************************************
void MOSES1::start(bool view, long int nb_eval){
  
  
  init();
  cout.precision(8);
  while ((f_min  >  f_arret) && (evaluations<nb_eval) )
    {  
      search_min_pop();
      nb_mutants=0;
      while(nb_mutants==0) nb_mutants=binom(_taille_pop-1,pmut);    
      mutation();
      selection();      
      if((evaluations % 10000) == 0) 
	if(view){
	  cout << _dim << " " << _taille_pop << " " << pmut << " " << rayon << " " << f_min << " " << nb_mutants << " " << evaluations << endl;


	}
      
    }
  
  
}

//***************************************************************************

void MOSES1::info(){



    S=0.0;
    S2=0.0;
    for (int j=0; j < _dim; j++) {S+= optimum[j];S2+=optimum[j]*optimum[j];}    
    S=S/(double)_dim;
    S2=sqrt(S2);

    cout.precision(4);
    // cout.setw(9);
  cout << "Minimizer :  " << "dimension du probleme    : " << _dim << endl;
  cout << "Minimizer :  " << "taille de la population  : " << _taille_pop << endl;
  cout << "Minimizer :  " << "arret                    : " << f_arret << endl;
  cout << "Minimizer :  " << "probabilit de mutation  : " << pmut <<  endl;
  cout << "Minimizer :  " << "Rayon                    : " << rayon << endl;
  cout << "Minimizer :  " << "nombre de mutants        : " << nb_mutants << endl;
  cout << "Minimizer :  " << "evaluations              : " << evaluations << endl;
  cout << "Minimizer :  " << "minimum                  : "<< f_min << endl;
  cout << "Minimizer :  " << "solution                 : " << endl;
  cout << "{ ";
  for (int j=0; j < _dim-1; j++) cout << optimum[j] << " ,  ";
  cout << optimum[_dim-1] << " }" << endl;
  cout << "Minimizer:  " <<  "distance de l'origine    : " << S2 << endl ;
  // cout <<  pop[_taille_pop-1][j] << " " ;
  cout << endl;

      
}



//**************************************************************************

void MOSES1::init(){
  values=new double [_taille_pop];
  optimum=new double [_dim];
  srand48(time(0));
  srand(time(0));

  pop=new (double *)[_taille_pop]; 
  for(int i=0;i<_taille_pop;i++) 
    {
      pop[i] = new double[_dim]; 
      for (int j=0;j<_dim;j++)
	pop[i][j] = _inf + (_sup - _inf)*d_alea(); 
    }

  for (int i = 0 ;i < _taille_pop ; i++) values[i] = f(pop[i]);
  for (int j = 0 ;j < _dim ; j++) optimum[j] = pop[_taille_pop-1][j];
  f_min = values[_taille_pop-1];
  // f_min=10;
  Imin = _taille_pop-1 ;
  nb_mutants=_taille_pop-1;

}

//**************************************************************

int MOSES1::search_min_pop(){  
  
  Imin=nb_mutants;

  for (int i = 0; i <= nb_mutants ; i++)
    if ( values[i]  <  f_min ) {Imin = i; f_min = values[i];} 
  
  if ( Imin < nb_mutants )
    for (int j = 0; j < _dim ; j++) optimum[j]=pop[Imin][j];   
  
  return(0);
} 

//***************************************************************

int MOSES1::binom(int n, double p){

  int k=0;
  for(int i=0;i<n;i++) if(d_alea()<p) k++;
  return(k);  
}

//***************************************************************

void MOSES1::mutation(){
 
  for (int i = 0 ; i < nb_mutants ; i++) 
    {
      int j = i_alea(_dim) ;
      double Mut = pop[i][j] +  rayon * (1.0 - 2.0 * d_alea());
      if ( Mut < _inf)  Mut = 2.0*_inf - Mut ;  
      else if ( Mut > _sup)  Mut = 2.0*_sup - Mut ;

      double *x1=new double[_dim];
      for(int k=0;k<_dim;k++) x1[k]=pop[i][k];
      x1[j]=Mut;
      
      values[i] = f(x1);
      delete [] x1;
      pop[i][j] = Mut ;
    }
}

//***************************************************************

void MOSES1::selection(){
 
  for (int i = nb_mutants ; i < _taille_pop ; i++)  
    {
      for(int j=0;j<_dim;j++) pop[i][j]=optimum[j]; 
      values[i] = f_min;
    }  
}

//***************************************************************

double  MOSES1::f(double *x){
  evaluations++;
        
 return _F->Eval(x);
}





// int MOSES1::mutation(){

// }


#endif
