#define __cplusplus__
#ifndef __MOSES3__
#define __MOSES3__

#include <cmath>
#include <iostream>
#include <cstdlib>
#include <ctime>

#include "Function.hpp"


#define deux_pi (2.0*M_PI)




class MOSES3
{
private:
  int _taille_pop;
  double _inf;
  double _sup;
  double **pop;
  double **pop1;
  int _dim;
  // double *optimum;
  double f_arret , S, S2;
  double pmut, rayon;
  Function* _F;
  double *optimum;
  double *values;
  double *values_tmp;
  int *permut;
  double f_min;
  double I_min;
  long int evaluations;
  int nb_mutants;
  int Imin;

public:

  MOSES3();
  MOSES3(Function *F,  int n);
  ~MOSES3();
  double getOptimum();
  void set_dim(int d);
  void set_Rad(double r);
  void set_Size(int n);
  void set_pmut(double p);
  void set_arret(double arr);
  void set_fmin(double fm);

  void start(bool view, long int nb_eval);
  // void minimum(double *v);
  void mutation();
  void sort();
  int intcompare(const void *i, const void *j);
  void init();
  double f(double *x);
  void info();
};


//**************************************************************************

MOSES3::MOSES3(){

  _taille_pop=8;
  f_arret=0.0001; 
  pmut=0.4;
  rayon=0.1;
  evaluations=0;
  _F=new Function();
  _inf=_F->getXmin();
  _sup=_F->getXmax();
  _dim=_F->get_dim();

}

//***************************************************************************************
MOSES3::MOSES3(Function* F, int n){

  _taille_pop=n;
  f_arret=0.0001; 
  pmut=0.4;
  rayon=0.1;
  evaluations=0;
  _F=F;
  _inf=_F->getXmin();
  _sup=_F->getXmax();
  _dim=_F->get_dim();
}

//**************************************************************************

MOSES3::~MOSES3(){
  delete [] values_tmp;
  delete [] values;
  delete [] optimum;
  for(int i=0;i<_taille_pop;i++) delete [] pop[i];
  delete [] pop;
  for(int i=0;i<_taille_pop;i++) delete [] pop1[i];
  delete [] pop1;
  delete [] permut;
  

}

//**************************************************************************

void MOSES3::set_dim(int d){
  _dim=d;
}

void MOSES3::set_Rad(double r){
  rayon=r;
}

void MOSES3::set_Size(int n){
  _taille_pop=n;
}

void MOSES3::set_pmut(double p){

  pmut=p;
}

void MOSES3::set_arret(double arr){

  f_arret=arr;
}

void MOSES3::set_fmin(double fm){

  f_min=fm;
}



//**************************************************************************
double MOSES3::getOptimum(){
  return(f_min);
}

//**************************************************************************
void MOSES3::start(bool view, long int nb_eval){
  
  
  init();
  cout.precision(8);
  while ((f_min  >  f_arret) && (evaluations<nb_eval) )
    {  
      sort();
      mutation();
      if((evaluations % 10000) == 0) 
	if(view){
	  cout << _dim << " " << _taille_pop << " " << pmut << " " << rayon << " " << f_min << " " << nb_mutants << " " << evaluations << endl;


	}
      
    }
  
  
}

//***************************************************************************

void MOSES3::info(){



    S=0.0;
    S2=0.0;
    for (int j=0; j < _dim; j++) {S+= optimum[j];S2+=optimum[j]*optimum[j];}    
    S=S/(double)_dim;
    S2=sqrt(S2);

    cout.precision(4);
    // cout.setw(9);
  cout << "Minimizer :  " << "dimension du probleme    : " << _dim << endl;
  cout << "Minimizer :  " << "taille de la population  : " << _taille_pop << endl;
  cout << "Minimizer :  " << "arret                    : " << f_arret << endl;
  cout << "Minimizer :  " << "probabilit de mutation  : " << pmut <<  endl;
  cout << "Minimizer :  " << "Rayon                    : " << rayon << endl;
  // cout << "Minimizer :  " << "nombre de mutants        : " << nb_mutants << endl;
  cout << "Minimizer :  " << "evaluations              : " << evaluations << endl;
  cout << "Minimizer :  " << "minimum                  : "<< f_min << endl;
  cout << "Minimizer :  " << "solution                 : " << endl;
  cout << "{ ";
  for (int j=0; j < _dim-1; j++) cout << optimum[j] << " ,  ";
  cout << optimum[_dim-1] << " }" << endl;
  cout << "Minimizer:  " <<  "distance de l'origine    : " << S2 << endl ;
  // cout <<  pop[_taille_pop-1][j] << " " ;
  cout << endl;

      
}



//**************************************************************************

void MOSES3::init(){
  values=new double [_taille_pop];
  values_tmp=new double [_taille_pop];
  permut=new int [_taille_pop];
  optimum=new double [_dim];
  srand48(time(0));
  srand(time(0));

  pop=new (double *)[_taille_pop]; 
  pop1=new (double *)[_taille_pop]; 
  for(int i=0;i<_taille_pop;i++) 
    {
      pop[i] = new double[_dim]; 
      pop1[i] = new double[_dim]; 
      for (int j=0;j<_dim;j++)
	pop[i][j] = _inf + (_sup - _inf)*d_alea(); 
    }

  for (int i = 0 ;i < _taille_pop ; i++) values[i] = f(pop[i]);
  for (int j = 0 ;j < _dim ; j++) optimum[j] = pop[_taille_pop-1][j];
  f_min = values[_taille_pop-1];
  // f_min=10;
  Imin = _taille_pop-1 ;
  nb_mutants=_taille_pop-1;

}

 
void MOSES3::sort(){  

  for (int i = 0 ; i < _taille_pop ; i++) permut[i]=i;
  // qsort(permut, _taille_pop, sizeof(int), intcompare);
  trier(permut,values,_taille_pop);
  for (int i = 0 ; i < _taille_pop ; i++){
    values_tmp[i]=values[permut[i]];
    for(int k=0;k<_dim;k++) pop1[i][k]=pop[permut[i]][k];
  }

  for (int i = 0 ; i < _taille_pop ; i++){
    values[i]=values_tmp[i];
    for(int k=0;k<_dim;k++) pop[i][k]=pop1[i][k];
    // cout << values[i] << " " ;
  }
  //  cout << endl;
  f_min=values[0];
} 

//***************************************************************

void MOSES3::mutation(){
 
  for (int i = 0 ; i < _taille_pop ; i++) 
    {
      if(d_alea()<pmut){
	int j = i_alea(_dim) ;
	double Mut = pop[i][j] +  rayon * (1.0 - 2.0 * d_alea());
	if ( Mut < _inf)  Mut = 2.0*_inf - Mut ;  
	else if ( Mut > _sup)  Mut = 2.0*_sup - Mut ;
	
	pop[i][j] = Mut ;
      	values[i] = f(pop[i]);
      
      }
      else{
	
	int j = rand() % (1+i);
	for(int k=0;k<_dim;k++) pop[i][k]=pop1[j][k];
	values[i]=values_tmp[j];
      }
    }
}


//***************************************************************

double  MOSES3::f(double *x){
  evaluations++;
  double y=_F->Eval(x);      
 return (y);
}





// int MOSES3::mutation(){

// }


#endif
