#define __cplusplus__
#ifndef __GENETIC__
#define __GENETIC__

#include <cmath>
#include <iostream>
#include <cstdlib>
#include <ctime>

// #include "fonction.hpp"


#define deux_pi (2.0*M_PI)


class Genetic
{
private:
  int _taille_pop;
  double _inf;
  double _sup;
  double **pop;
  int _dim;
  // double *optimum;
  double f_arret , S, S2;
  double pmut, rayon;
  // Fonction& _Fonc;
  double *optimum;
  double *values;
  double f_min;
  double I_min;
  long int evaluations;
  int nb_mutants;
  int Imin;

public:

  Genetic();
  Genetic(int d, int n);
  ~Genetic();
  double getOptimum();

  void set_dim(int d);
  void set_Rad(double r);
  void set_Size(int n);
  void set_pmut(double p);
  void set_arret(double arr);

  void start(bool view, long int nb_eval);
  // void minimum(double *v);
  void mutation();
  void selection();
  int binom(int n, double p);
  void init();
  double f(double *x);
  double Delta(double *x,double y,int j);
  int search_min_pop();
  double d_alea();
  int i_alea(int d);
  void info();
};


//**************************************************************************

Genetic::Genetic(){

  _taille_pop=8;
  _inf=-5.12;
  _sup=5.12;
  _dim=10;
  // double *optimum;
  f_arret=0.0001; 
  // , S, S2;
  pmut=0.4;
  rayon=0.1;
  // _Fonc=*new Fonction();
  // double *optimum;
  // double *values;
  // double f_min,f_max;
  evaluations=0;


}

//*********************************************************************************************************************************
Genetic::Genetic(int d, int n){

  _taille_pop=n;
  _inf=-5.12;
  _sup=5.12;
  _dim=d;
  // double *optimum;
  f_arret=0.0001; 
  // , S, S2;
  pmut=0.4;
  rayon=0.1;
  // _Fonc=*new Fonction();
  // double *optimum;
  // double *values;
  // double f_min,f_max;
  evaluations=0;


}

//**************************************************************************

Genetic::~Genetic(){
  delete values;
  delete optimum;
  for(int i=0;i<_taille_pop;i++) delete pop[i];
  delete pop;
  

}

//**************************************************************************

void Genetic::set_dim(int d){
  _dim=d;
}

void Genetic::set_Rad(double r){
  rayon=r;
}

void Genetic::set_Size(int n){
  _taille_pop=n;
}

void Genetic::set_pmut(double p){

  pmut=p;
}

void Genetic::set_arret(double arr){

  f_arret=arr;
}



//**************************************************************************
double Genetic::getOptimum(){
  return(f_min);
}

//**************************************************************************
void Genetic::start(bool view, long int nb_eval){
  
  
  init();
  cout.precision(8);
  while ((f_min  >  f_arret) && (evaluations<nb_eval) )
    {  
      search_min_pop();
      nb_mutants=0;
      while(nb_mutants==0) nb_mutants=binom(_taille_pop-1,pmut);    
      mutation();
      selection();      
      if((evaluations % 10000) == 0) 
	if(view){
	  cout << _dim << " " << _taille_pop << " " << pmut << " " << rayon << " " << f_min << " " << nb_mutants << " " << evaluations << endl;


	}
      
    }
  
  
}

//***************************************************************************

void Genetic::info(){



   S=0.0;
   S2=0.0;
   for (int j=0; j < _dim; j++) {S+= optimum[j];S2+=optimum[j]*optimum[j];}    
   S=S/(double)_dim;
   S2=sqrt(S2);

  
  cout << "Minimizer :  " << "dimension du probleme    : " << _dim << endl;
  cout << "Minimizer :  " << "taille de la population  : " << _taille_pop << endl;
  cout << "Minimizer :  " << "arret                    : " << f_arret << endl;
  cout << "Minimizer :  " << "probabilit de mutation  : " << pmut <<  endl;
  cout << "Minimizer :  " << "Rayon                    : " << rayon << endl;
  cout << "Minimizer :  " << "nombre de mutants        : " << nb_mutants << endl;
  cout << "Minimizer :  " << "evaluations              : " << evaluations << endl;
  cout << "Minimizer :  " <<"minimum                  : "<< f_min << endl;
  cout << "solution : " << S << " " <<  S2 << endl; 
  // cout <<  pop[_taille_pop-1][j] << " " ;
  cout << endl;

      
}



//**************************************************************************

void Genetic::init(){
  values=new double [_taille_pop];
  optimum=new double [_dim];
  srand48(time(0));
  srand(time(0));

  pop=new (double *)[_taille_pop]; 
  for(int i=0;i<_taille_pop;i++) 
    {
      pop[i] = new double[_dim]; 
      for (int j=0;j<_dim;j++)
	pop[i][j] = _inf + (_sup - _inf)*d_alea(); 
    }

  for (int i = 0 ;i < _taille_pop ; i++) values[i] = f(pop[i]);
  for (int j = 0 ;j < _dim ; j++) optimum[j] = pop[_taille_pop-1][j];
  f_min = values[_taille_pop-1];
  // f_min=10;
  Imin = _taille_pop-1 ;
  nb_mutants=_taille_pop-1;

}

//**************************************************************

int Genetic::search_min_pop(){  
  
  Imin=nb_mutants;

  for (int i = 0; i <= nb_mutants ; i++)
    if ( values[i]  <  f_min ) {Imin = i; f_min = values[i];} 
  
  if ( Imin < nb_mutants )
    for (int j = 0; j < _dim ; j++) optimum[j]=pop[Imin][j];   
  
  return(0);
} 

//***************************************************************

int Genetic::binom(int n, double p){

  int k=0;
  for(int i=0;i<n;i++) if(d_alea()<p) k++;
  return(k);  
}

//***************************************************************

void Genetic::mutation(){
 
  for (int i = 0 ; i < nb_mutants ; i++) 
    {
      int j = i_alea(_dim) ;
      double Mut = pop[i][j] +  rayon * (1.0 - 2.0 * d_alea());
      if ( Mut < _inf)  Mut = 2.0*_inf - Mut ;  
      else if ( Mut > _sup)  Mut = 2.0*_sup - Mut ;
      
      values[i] += Delta(pop[i],Mut ,j);
      pop[i][j] = Mut ;
    }
}

//***************************************************************

void Genetic::selection(){
 
  for (int i = nb_mutants ; i < _taille_pop ; i++)  
    {
      for(int j=0;j<_dim;j++) pop[i][j]=optimum[j]; 
      values[i] = f_min;
    }  
}

//***************************************************************

double  Genetic::f(double *x){
  double E;
  
  E = 0.0 ;
  for (int j = 0; j < _dim ; j++)
     E += x[j]*x[j]  - 10.0 * cos(deux_pi * x[j]) + 10.0 ;
  evaluations++;
        
 return E;
}


//***************************************************************

double Genetic::Delta(double *x,double y,int j){
  double D1,D2;
  
  D1 = x[j]*x[j]  - 10.0 * cos(deux_pi * x[j]);
  D2 = y*y  - 10.0 * cos(deux_pi * y);
  evaluations++;

 return D2-D1;
}


//**************************************************************************

double Genetic::d_alea(){

  return(drand48());
}


//*************************************************************************

int Genetic::i_alea(int d){
  return(rand() % d);
}

// int Genetic::mutation(){

// }


#endif
